@@ -28,6 +28,8 @@ from base64 import standard_b64encode
 
 import bz2
 import os
+import shutil
+import tempfile
 import tarfile
 import zipfile
 
@@ -356,23 +358,30 @@ class UScanSourceTests(TestCaseWithTransport):
 
     def test_export_watchfile_none(self):
         src = UScanSource(self.tree, False)
-        self.assertRaises(NoSuchFile, src._export_file, 'watch')
+        self.assertRaises(NoSuchFile, src._export_file, 'watch', self.test_dir)
 
     def test_export_watchfile_top_level(self):
         src = UScanSource(self.tree, True)
         self.build_tree(['watch'])
         self.tree.add(['watch'])
-        self.assertIsNot(src._export_file('watch'), None)
+        tmpdir = tempfile.mkdtemp()
+        try:
+            self.assertIsNot(src._export_file('watch', tmpdir), None)
+        finally:
+            shutil.rmtree(tmpdir)
 
     def test_export_watchfile(self):
         src = UScanSource(self.tree, False)
         self.build_tree(['debian/', 'debian/watch'])
         self.tree.smart_add(['debian/watch'])
-        self.assertIsNot(src._export_file('watch'), None)
+        tmpdir = tempfile.mkdtemp()
+        try:
+            self.assertIsNot(src._export_file('watch', tmpdir), None)
+        finally:
+            shutil.rmtree(tmpdir)
 
     def test__xml_report_extract_upstream_version(self):
-        self.assertEquals("1.2.9",
-            UScanSource._xml_report_extract_upstream_version("""
+        text = b"""
 <dehs>
 <package>tdb</package>
 <debian-uversion>1.2.8</debian-uversion>
@@ -380,41 +389,66 @@ class UScanSourceTests(TestCaseWithTransport):
 <upstream-version>1.2.9</upstream-version>
 <upstream-url>ftp://ftp.samba.org/pub/tdb/tdb-1.2.9.tar.gz</upstream-url>
 <status>Newer version available</status>
-</dehs>"""))
+</dehs>"""
+        self.assertEquals("1.2.9",
+            UScanSource._xml_report_extract_upstream_version(text))
 
     def test__xml_report_extract_upstream_version_warnings(self):
-        self.assertIs(None,
-            UScanSource._xml_report_extract_upstream_version("""
+        text = b"""
 <dehs>
 <package>tdb</package>
 <warnings>uscan warning: Unable to determine current version
 in debian/watch, skipping:
 ftp://ftp.samba.org/pub/tdb/tdb-(.+).tar.gz</warnings>
 </dehs>
-"""))
+"""
+        self.assertIs(
+            None,
+            UScanSource._xml_report_extract_upstream_version(text))
+
+    def test__xml_report_extract_upstream_version_noise(self):
+        text = b"""
+<dehs>
+blahf =>
+<package>tdb</package>
+<debian-uversion>1.2.8</debian-uversion>
+<upstream-version>1.2.9</upstream-version>
+<warnings>uscan warning: Unable to determine current version
+in debian/watch, skipping:
+ftp://ftp.samba.org/pub/tdb/tdb-(.+).tar.gz</warnings>
+</dehs>
+"""
+        self.assertEquals("1.2.9",
+            UScanSource._xml_report_extract_upstream_version(text))
 
 
 class GuessUpstreamRevspecTests(TestCase):
 
     def test_guess_upstream_revspec(self):
         self.assertEqual(
-                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2'],
+                [u'tag:1.2', u'tag:foo-1.2', u'tag:v1.2', u'tag:v.1.2'],
                 list(guess_upstream_revspec('foo', '1.2')))
 
     def test_snapshot(self):
         self.assertEqual(
                 [u'revno:1200', u'tag:1.2+bzr1200', u'tag:foo-1.2+bzr1200',
-                 u'tag:v1.2+bzr1200'],
+                 u'tag:v1.2+bzr1200', u'tag:v.1.2+bzr1200'],
                 list(guess_upstream_revspec('foo', '1.2+bzr1200')))
         self.assertEqual(
                 [u'revno:1200', u'tag:1.2~bzr1200', u'tag:foo-1.2~bzr1200',
-                 u'tag:v1.2~bzr1200'],
+                 u'tag:v1.2~bzr1200', u'tag:v.1.2~bzr1200'],
                 list(guess_upstream_revspec('foo', '1.2~bzr1200')))
         self.assertEqual(
                 [u'git:abc1def', u'tag:1.2~git20180101.abc1def',
                  u'tag:foo-1.2~git20180101.abc1def',
-                 u'tag:v1.2~git20180101.abc1def'],
+                 u'tag:v1.2~git20180101.abc1def',
+                 u'tag:v.1.2~git20180101.abc1def'],
                 list(guess_upstream_revspec('foo', '1.2~git20180101.abc1def')))
+        self.assertEqual(
+                [u'date:2018-01-01', u'tag:1.2~git20180101',
+                 u'tag:foo-1.2~git20180101',
+                 u'tag:v1.2~git20180101', u'tag:v.1.2~git20180101'],
+                list(guess_upstream_revspec('foo', '1.2~git20180101')))
 
 
 class UpstreamBranchSourceTests(TestCaseWithTransport):
@@ -661,6 +695,9 @@ class TestUpstreamVersionAddRevision(TestCaseWithTransport):
     def revision_id_to_revno(self, revid):
         return self.revnos[revid]
 
+    def revision_id_to_dotted_revno(self, revid):
+        return (self.revnos[revid], )
+
     def get_revision(self, revid):
         rev = Revision(revid)
         if revid in self.svn_revnos:
@@ -714,6 +751,18 @@ class TestUpstreamVersionAddRevision(TestCaseWithTransport):
             "1.3+git20180101.e7f47cf",
             upstream_version_add_revision(
                 self, "1.3", b"somegitrev"))
+        self.assertEquals(
+            "1.0~git20180101",
+            upstream_version_add_revision(
+                self, "1.0~git20160320", b"somegitrev"))
+        self.assertEquals(
+            "1.0~git20180101.1.e7f47cf",
+            upstream_version_add_revision(
+                self, "1.0~git20180101.0.11b1d57", b"somegitrev"))
+        self.assertEquals(
+            "1.0~git20180101.0.e7f47cf",
+            upstream_version_add_revision(
+                self, "1.0~git20170101.0.11b1d57", b"somegitrev"))
 
 
 class GetExportUpstreamRevisionTests(TestCase):
@@ -848,6 +897,19 @@ class PristineTarSourceTests(TestCaseWithTransport):
         root_id = self.tree.path2id("")
         self.source = PristineTarSource(self.tree.branch)
 
+    def test_gbp_tag_format(self):
+        self.build_tree_contents([
+            ('unstable/debian/', ),
+            ('unstable/debian/gbp.conf', """\
+[DEFAULT]
+upstream-tag = blah-%(version)s
+""")])
+        self.source = PristineTarSource.from_tree(
+            self.tree.branch, self.tree)
+        upstream_v_no = "0.1"
+        self.assertEqual(self.source.tag_name(upstream_v_no),
+                "blah-" + upstream_v_no)
+
     def test_upstream_tag_name(self):
         upstream_v_no = "0.1"
         self.assertEqual(self.source.tag_name(upstream_v_no),
@@ -858,12 +920,28 @@ class PristineTarSourceTests(TestCaseWithTransport):
                 "upstream-ubuntu-0.3")
 
     def test_version(self):
-        self.assertEquals(['upstream-3.3', 'upstream-debian-3.3',
-            'upstream-ubuntu-3.3', 'upstream/3.3', 'upstream_3.3'],
+        self.assertEquals([
+            'upstream-3.3',
+            'upstream-debian-3.3',
+            'upstream-ubuntu-3.3', 'upstream/3.3', 'upstream-3.3',
+            'upstream_3.3'],
             self.source.possible_tag_names("3.3", component=None))
 
+    def test_version_with_tilde(self):
+        self.assertEquals([
+            'upstream-3.3~brz232',
+            'upstream-debian-3.3~brz232',
+            'upstream-ubuntu-3.3~brz232',
+            'upstream/3.3~brz232',
+            'upstream-3.3~brz232',
+            'upstream/3.3_brz232',
+            'upstream_3.3~brz232'],
+            self.source.possible_tag_names("3.3~brz232", component=None))
+
     def test_version_component(self):
-        self.assertEquals(['upstream-3.3/extlib', 'upstream-debian-3.3/extlib',
+        self.assertEquals([
+            'upstream-3.3/extlib',
+            'upstream-debian-3.3/extlib',
             'upstream-ubuntu-3.3/extlib'],
             self.source.possible_tag_names("3.3", component="extlib"))
 
