@@ -22,11 +22,11 @@ import os
 import re
 import shutil
 import subprocess
+import sys
 import tarfile
 import tempfile
 
-from debian.changelog import Version
-
+from ....errors import NoSuchFile
 from .... import osutils
 from ....export import export
 from ....trace import (
@@ -37,7 +37,6 @@ from ....trace import (
 from ..errors import (
     MissingUpstreamTarball,
     PackageVersionNotPresent,
-    WatchFileMissing,
     )
 from ..repack_tarball import (
     get_filetype,
@@ -174,76 +173,89 @@ class UScanSource(UpstreamSource):
         self.tree = tree
         self.top_level = top_level
 
-    def _export_watchfile(self):
+    def _export_file(self, name, directory):
         if self.top_level:
-            watchfile = 'watch'
+            file = name
         else:
-            watchfile = 'debian/watch'
-        watch_id = self.tree.path2id(watchfile)
-        if watch_id is None:
-            raise WatchFileMissing()
-        (tmp, tempfilename) = tempfile.mkstemp()
-        try:
-            tmp = os.fdopen(tmp, 'wb')
-            watch = self.tree.get_file_text(watchfile, watch_id)
-            tmp.write(watch)
-        finally:
-            tmp.close()
-        return tempfilename
+            file = 'debian/' + name
+        if not self.tree.has_filename(file):
+            raise NoSuchFile(file, self.tree)
+        output_path = os.path.join(directory, 'debian', name)
+        output_dir = os.path.dirname(output_path)
+        if not os.path.isdir(output_dir):
+            os.makedirs(output_dir)
+        with open(output_path, 'wb') as f:
+            f.write(self.tree.get_file_text(file))
+        return output_path
+
+    @staticmethod
+    def _run_dehs_uscan(args, cwd):
+        p = subprocess.Popen(
+            ["uscan", "--dehs"] + args,
+            stdout=subprocess.PIPE, cwd=cwd)
+        (stdout, stderr) = p.communicate()
+        return stdout, p.returncode
+
+    @staticmethod
+    def _xml_report_extract_warnings(text):
+        from xml.sax.saxutils import unescape
+        for m in re.finditer(b"<warnings>(.*)</warnings>", text):
+            warning(unescape(m.group(1).decode()))
 
     @staticmethod
     def _xml_report_extract_upstream_version(text):
-        from xml.dom.minidom import parseString
-        dom = parseString(text)
-        dehs_tags = dom.getElementsByTagName("dehs")
-        if len(dehs_tags) != 1:
+        UScanSource._xml_report_extract_warnings(text)
+        from xml.sax.saxutils import unescape
+        # uscan --dehs's output isn't well-formed XML, so let's fall back to
+        # regexes instead..
+        m = re.search(b'<upstream-version>(.*)</upstream-version>', text)
+        if not m:
             return None
-        dehs_tag = dehs_tags[0]
-        for w in dehs_tag.getElementsByTagName("warnings"):
-            warning(w.firstChild.wholeText)
-        upstream_version_tags = dehs_tag.getElementsByTagName("upstream-version")
-        if len(upstream_version_tags) != 1:
-            return None
-        upstream_version_tag = upstream_version_tags[0]
-        return upstream_version_tag.firstChild.wholeText.encode("utf-8")
+        return unescape(m.group(1).decode())
+
+    @staticmethod
+    def _xml_report_extract_target_paths(text):
+        UScanSource._xml_report_extract_warnings(text)
+        from xml.sax.saxutils import unescape
+        return [
+            unescape(m.group(1).decode())
+            for m in re.finditer(b'<target-path>(.*)</target-path>', text)]
 
     def get_latest_version(self, package, current_version):
+        tmpdir = tempfile.mkdtemp()
         try:
-            tempfilename = self._export_watchfile()
-        except WatchFileMissing:
-            note("No watch file to use to check latest upstream release.")
-            return None
-        try:
-            p = subprocess.Popen(["uscan", "--package=%s" % package, "--report",
-                "--no-download", "--dehs", "--watchfile=%s" % tempfilename,
-                "--upstream-version=%s" % current_version],
-                stdout=subprocess.PIPE)
-            (stdout, stderr) = p.communicate()
+            try:
+                watch_tempfilename = self._export_file('watch', tmpdir)
+            except NoSuchFile:
+                note("No watch file to use to check latest upstream release.")
+                return None
+            args = ["--watchfile=%s" % watch_tempfilename,
+                    "--package=%s" % package, "--report",
+                    "--no-download", "--upstream-version=%s" % current_version]
+            text, retcode = self._run_dehs_uscan(args, cwd=tmpdir)
         finally:
-            os.unlink(tempfilename)
-        return self._xml_report_extract_upstream_version(stdout)
+            shutil.rmtree(tmpdir)
+        return self._xml_report_extract_upstream_version(text)
 
     def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using uscan to look for the upstream tarball.")
+        tmpdir = tempfile.mkdtemp()
         try:
-            tempfilename = self._export_watchfile()
-        except WatchFileMissing:
-            note("No watch file to use to retrieve upstream tarball.")
-            raise PackageVersionNotPresent(package, version, self)
-        try:
-            r = subprocess.call(["uscan", "--watchfile=%s" % tempfilename, 
-                "--upstream-version=%s" % version,
-                "--force-download", "--rename", "--package=%s" % package,
-                "--check-dirname-level=0",
-                "--download", "--destdir=%s" % target_dir,
-                "--download-version=%s" % version])
+            # Just export all of debian/, since e.g. uupdate needs more of it.
+            export(self.tree, os.path.join(tmpdir, 'debian'), format='dir',
+                   subdir='debian')
+            args = ["--force-download", "--rename",
+                    "--check-dirname-level=0",
+                    "--download", "--destdir=%s" % target_dir,
+                    "--download-version=%s" % version]
+            text, r = self._run_dehs_uscan(args, cwd=tmpdir)
+            orig_files = self._xml_report_extract_target_paths(text)
+            if not orig_files:
+                note("uscan could not find the needed tarball.")
+                raise PackageVersionNotPresent(package, version, self)
         finally:
-            os.unlink(tempfilename)
-        if r != 0:
-            note("uscan could not find the needed tarball.")
-            raise PackageVersionNotPresent(package, version, self)
-        orig_files = gather_orig_files(package, version, target_dir)
-        if orig_files is None:
+            shutil.rmtree(tmpdir)
+        if not orig_files:
             note("the expected files generated by uscan could not be found in"
                  "%s.", target_dir)
             raise PackageVersionNotPresent(package, version, self)
@@ -319,11 +331,13 @@ def gather_orig_files(package, version, path):
     :param version: package upstream version string
     :return: List of orig tarfile paths, or None if none were found
     """
-    prefix = "%s_%s.orig" % (package.encode('ascii'), version.encode('ascii'))
+    prefix = ("%s_%s.orig" % (package, version))
     ret = []
     path = os.path.abspath(path)
     if not os.path.isdir(path):
         return None
+    if isinstance(path, bytes):
+        prefix = prefix.encode(osutils._fs_enc)
     for filename in os.listdir(path):
         if filename.endswith('.asc'):
             continue
@@ -434,11 +448,11 @@ def extract_tarball_version(path, packagename):
     # Debian style tarball
     m = re.match(packagename+"_(.*).orig", basename)
     if m:
-        return str(m.group(1))
+        return m.group(1)
     # Traditional, PACKAGE-VERSION.tar.gz
     m = re.match(packagename+"-(.*)", basename)
     if m:
-        return str(m.group(1))
+        return m.group(1)
     return None
 
 
@@ -516,20 +530,14 @@ class LaunchpadReleaseFileSource(UpstreamSource):
         hosted_file = release_files[0]
         tmpdir = tempfile.mkdtemp(prefix="builddeb-launchpad-source-")
         try:
-            inf = hosted_file.open()
-            try:
+            with hosted_file.open() as inf:
                 note("Downloading upstream tarball %s from Launchpad",
                      inf.filename)
                 filename = inf.filename.encode(osutils._fs_enc)
                 filename = filename.replace("/", "")
                 tmppath = os.path.join(tmpdir, filename)
-                outf = open(tmppath, 'wb')
-                try:
+                with open(tmppath, 'wb') as outf:
                     outf.write(inf.read())
-                finally:
-                    outf.close()
-            finally:
-                inf.close()
             dest_name = new_tarball_name(package, version, filename)
             repack_tarball(tmppath, dest_name, target_dir=target_dir)
             return os.path.join(target_dir, dest_name)
@@ -541,4 +549,4 @@ class LaunchpadReleaseFileSource(UpstreamSource):
         for release in self.project_series.releases:
             versions.append((release.date_released, release.version))
         versions.sort()
-        return versions[-1][1].encode("utf-8")
+        return versions[-1][1]
