@@ -152,11 +152,12 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
     :param version_string: Original version string
     :param revid: Revision id of the revision
     """
-    revno = upstream_branch.revision_id_to_revno(revid)
+    revno = upstream_branch.revision_id_to_dotted_revno(revid)
+    revno_str = '.'.join(map(str, revno))
 
     m = re.match(r"^(.*)([\+~])bzr(\d+)$", version_string)
     if m:
-        return "%s%sbzr%d" % (m.group(1), m.group(2), revno)
+        return "%s%sbzr%s" % (m.group(1), m.group(2), revno_str)
 
     rev = upstream_branch.repository.get_revision(revid)
     gitid = extract_gitid(rev)
@@ -169,6 +170,19 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
     if m and gitid:
         return "%s%sgit%s.%s" % (m.group(1), m.group(2), gitdate, gitid)
 
+    m = re.match(r"^(.*)([\+~])git(\d{8})\.(\d+)\.([a-f0-9]{7})$", version_string)
+    if m and gitid:
+        if gitdate == m.group(3):
+            snapshot = int(m.group(4)) + 1
+        else:
+            snapshot = 0
+        return "%s%sgit%s.%d.%s" % (
+            m.group(1), m.group(2), gitdate, snapshot, gitid)
+
+    m = re.match(r"^(.*)([\+~])git(\d{8})$", version_string)
+    if m and gitid:
+        return "%s%sgit%s" % (m.group(1), m.group(2), gitdate)
+
     svn_revno = extract_svn_revno(rev)
 
     m = re.match(r"^(.*)([\+~])svn(\d+)$", version_string)
@@ -181,7 +195,7 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
     elif gitid:
         return "%s+git%s.%s" % (version_string, gitdate, gitid)
     else:
-        return "%s+bzr%d" % (version_string, revno)
+        return "%s+bzr%s" % (version_string, revno_str)
 
 
 def get_snapshot_revision(upstream_version):
@@ -261,12 +275,18 @@ def guess_upstream_revspec(package, version):
     if "+bzr" in version or "~bzr" in version:
         yield "revno:%s" % re.match(".*[~+]bzr(\\d+).*", version).group(1)
     if "+git" in version or "~git" in version:
-        yield "git:%s" % re.match(
-            ".*[~+]git([0-9]{8})\.([0-9a-f]{7}).*", version).group(2)
+        m = re.match(".*[~+]git([0-9]{8})\\.([0-9a-f]{7}).*", version)
+        if m:
+            yield "git:%s" % m.group(2)
+        else:
+            m = re.match(".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2}).*", version)
+            if m:
+                yield "date:%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
 
     yield 'tag:%s' % version
     yield 'tag:%s-%s' % (package, version)
     yield 'tag:v%s' % version
+    yield 'tag:v.%s' % version
 
 
 # TODO(jelmer): Move this into breezy.
@@ -283,6 +303,9 @@ class MinimalMemoryBranch(bzr_branch.Branch, _RelockDebugMixin):
             self.tags = DisabledTags(self)
         self._partial_revision_history_cache = []
         self._last_revision_info_cache = None
+        self._revision_id_to_revno_cache = None
+        self._partial_revision_id_to_revno_cache = {}
+        self._partial_revision_history_cache = []
 
     def lock_read(self):
         self.repository.lock_read()
