@@ -23,7 +23,9 @@ from base64 import (
     standard_b64decode,
     standard_b64encode,
     )
+import configparser
 import errno
+from io import TextIOWrapper
 import os
 import shutil
 import subprocess
@@ -43,6 +45,8 @@ from .... import (
     osutils,
     revision as _mod_revision,
     )
+from ....commit import NullCommitReporter
+from ....config import ConfigObj
 from ....errors import (
     BzrError,
     NoSuchRevision,
@@ -51,7 +55,6 @@ from ....errors import (
     NotBranchError,
     )
 from ....sixish import (
-    text_type,
     viewitems,
     )
 from ....trace import (
@@ -76,11 +79,7 @@ class PristineTarDeltaAbsent(PristineTarError):
     _fmt = 'There is not delta present for %(version)s.'
 
 
-def git_store_pristine_tar(controldir, filename, tree_id, delta, force=False):
-    try:
-        branch = controldir.open_branch(name='pristine-tar')
-    except NotBranchError:
-        branch = controldir.create_branch(name='pristine-tar')
+def git_store_pristine_tar(branch, filename, tree_id, delta, force=False):
     tree = branch.create_memorytree()
     with tree.lock_write():
         id_filename = '%s.id' % filename
@@ -95,12 +94,16 @@ def git_store_pristine_tar(controldir, filename, tree_id, delta, force=False):
                 # Nothing to do.
                 return
             if not force:
-                raise PristineTarError("An existing pristine tar entry exists for %s" %
-                        filename)
+                raise PristineTarError(
+                    "An existing pristine tar entry exists for %s" % filename)
         tree.put_file_bytes_non_atomic(id_filename, tree_id + b'\n')
         tree.put_file_bytes_non_atomic(delta_filename, delta)
         tree.add([id_filename, delta_filename], [None, None], ['file', 'file'])
-        tree.commit('Add pristine tar data for %s' % filename)
+        revid = tree.commit(
+            'Add pristine tar data for %s' % filename,
+            reporter=NullCommitReporter())
+        mutter('Added pristine tar data for %s: %s',
+               filename, revid)
 
 
 def reconstruct_pristine_tar(dest, delta, dest_filename):
@@ -149,7 +152,7 @@ def make_pristine_tar_delta(dest, tarball_path):
             raise
     (stdout, stderr) = proc.communicate()
     if proc.returncode != 0:
-        if 'excessively large binary delta' in stderr:
+        if b'excessively large binary delta' in stderr:
             raise PristineTarDeltaTooLarge(stderr)
         else:
             raise PristineTarError("Generating delta from tar failed: %s" %
@@ -168,7 +171,7 @@ def make_pristine_tar_delta_from_tree(tree, tarball_path, subdir=None, exclude=N
         except PristineTarDeltaTooLarge:
             raise
         except PristineTarError: # I.e. not PristineTarDeltaTooLarge
-            conf = debuild_config(tree, True)
+            conf = debuild_config(tree)
             if conf.debug_pristine_tar:
                 revno, revid = tree.branch.last_revision_info()
                 preserved = osutils.pathjoin(osutils.dirname(tarball_path),
@@ -222,12 +225,48 @@ def revision_pristine_tar_format(rev):
 class PristineTarSource(UpstreamSource):
     """Source that uses the pristine-tar revisions in the packaging branch."""
 
-    def __init__(self, branch):
+    def __init__(self, branch, tag_format=None, pristine_tar=None):
         self.branch = branch
+        self.tag_format = tag_format
+        self.pristine_tar = pristine_tar
 
     def __repr__(self):
         return "<%s at %s>" % (self.__class__.__name__, self.branch.base)
 
+    @classmethod
+    def from_tree(cls, branch, tree):
+        if tree and tree.has_filename('debian/gbp.conf'):
+            parser = configparser.ConfigParser(defaults={
+                'pristine-tar': 'false',
+                'upstream-tag': 'upstream/%(version)s'},
+                strict=False)
+            parser.read_string(
+                tree.get_file_text('debian/gbp.conf').decode(
+                    'utf-8', errors='replace'),
+                'debian/gbp.conf')
+            try:
+                tag_format = parser.get(
+                    'import-orig', 'upstream-tag', raw=True)
+            except configparser.Error:
+                try:
+                    tag_format = parser.get(
+                        'DEFAULT', 'upstream-tag', raw=True)
+                except configparser.Error:
+                    tag_format = None
+            try:
+                pristine_tar = parser.getboolean(
+                    'import-orig', 'pristine-tar')
+            except configparser.Error:
+                try:
+                    pristine_tar = parser.getboolean(
+                        'DEFAULT', 'pristine-tar')
+                except configparser.Error:
+                    pristine_tar = None
+        else:
+            tag_format = None
+            pristine_tar = None
+        return cls(branch, tag_format, pristine_tar)
+
     def tag_name(self, version, component=None, distro=None):
         """Gets the tag name for the upstream part of version.
 
@@ -237,12 +276,14 @@ class PristineTarSource(UpstreamSource):
         :param distro: Optional distribution name
         :return: a String with the name of the tag.
         """
+        if self.tag_format is not None:
+            return self.tag_format % {'version': version}
         if getattr(self.branch.repository, '_git', None):
             # In git, the convention is to use a slash
             if distro is None:
-                name = "upstream/" + version
+                name = "upstream/" + version.replace('~', '_')
             else:
-                name = "upstream-%s/%s" % (distro, version)
+                name = "upstream-%s/%s" % (distro, version.replace('~', '_'))
         else:
             if distro is None:
                 name = "upstream-" + version
@@ -273,7 +314,8 @@ class PristineTarSource(UpstreamSource):
 
     def import_component_tarball(self, package, version, tree, parent_ids,
             component=None, md5=None, tarball=None, author=None, timestamp=None,
-            subdir=None, exclude=None):
+            subdir=None, exclude=None, force_pristine_tar=False,
+            committer=None):
         """Import a tarball.
 
         :param package: Package name
@@ -281,6 +323,9 @@ class PristineTarSource(UpstreamSource):
         :param parent_ids: Dictionary mapping component names to revision ids
         :param component: Component name (None for base)
         :param exclude: Exclude directories
+        :param force_pristine_tar: Whether to force creating a pristine-tar
+            branch if one does not exist.
+        :param committer: Committer identity to use
         """
         if exclude is None:
             exclude = []
@@ -336,13 +381,13 @@ class PristineTarSource(UpstreamSource):
         with tree.lock_write():
             builder = tree.branch.get_commit_builder(
                     parents=parent_ids, revprops=revprops, timestamp=timestamp,
-                    timezone=timezone)
+                    timezone=timezone, committer=committer)
             try:
                 changes = [c for c in tree.iter_changes(basis_tree) if
                            include_change(c)]
                 list(builder.record_iter_changes(tree, base_revid, changes))
                 builder.finish_inventory()
-            except:
+            except BaseException:
                 builder.abort()
                 raise
             revid = builder.commit(message)
@@ -352,8 +397,22 @@ class PristineTarSource(UpstreamSource):
         if git_delta is not None:
             revtree = tree.branch.repository.revision_tree(revid)
             tree_id = revtree._lookup_path(u'')[2]
-            git_store_pristine_tar(self.branch.controldir,
-                    os.path.basename(tarball), tree_id, git_delta)
+            try:
+                pristine_tar_branch = self.branch.controldir.open_branch(
+                    name='pristine-tar')
+            except NotBranchError:
+                if force_pristine_tar:
+                    note('Creating new pristine-tar branch.')
+                    pristine_tar_branch = self.branch.controldir.create_branch(
+                        name='pristine-tar')
+                else:
+                    note('Not storing pristine-tar metadata, '
+                         'since there is no pristine-tar branch.')
+                    pristine_tar_branch = None
+            if pristine_tar_branch:
+                git_store_pristine_tar(
+                    pristine_tar_branch, os.path.basename(tarball), tree_id,
+                    git_delta)
         mutter(
             'imported %s version %s component %r as revid %s, tagged %s',
             package, version, component, revid, tag_name)
@@ -457,9 +516,12 @@ class PristineTarSource(UpstreamSource):
                 ]
         if component is None:
             # compatibility with git-buildpackage
-            tags += ["upstream/%s" % version]
+            tags.append("upstream/%s" % version)
+            tags.append("upstream-%s" % version)
+            if "~" in version:
+                tags.append("upstream/%s" % version.replace("~", "_"))
             # compatibility with svn-buildpackage
-            tags += ["upstream_%s" % version]
+            tags.append("upstream_%s" % version)
         return tags
 
     def get_pristine_tar_delta(self, package, version, dest_filename, revid=None):
@@ -539,6 +601,7 @@ def upstream_tag_version(tag):
             tag = tag[len('debian-'):]
         elif tag.startswith('ubuntu-'):
             tag = tag[len('ubuntu-'):]
+    tag = tag.replace('_', '~')
     if not '/' in tag:
         return (None, tag)
     (version, component) = tag.rsplit('/', 1)
